@@ -1,7 +1,7 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: January 14, 2018
+# Last Change: January 20, 2018
 # URL: https://humanfriendly.readthedocs.io
 
 """The main module of the `humanfriendly` package."""
@@ -39,7 +39,7 @@ from humanfriendly.prompts import prompt_for_choice  # NOQA
 from humanfriendly.compat import is_string, monotonic
 
 # Semi-standard module versioning.
-__version__ = '4.7'
+__version__ = '4.8'
 
 # Spinners are redrawn at most this many seconds.
 minimum_spinner_interval = 0.2
@@ -52,6 +52,7 @@ erase_line_code = '\r\x1b[K'
 hide_cursor_code = '\x1b[?25l'
 show_cursor_code = '\x1b[?25h'
 
+# Named tuples to define units of size.
 SizeUnit = collections.namedtuple('SizeUnit', 'divider, symbol, name')
 CombinedUnit = collections.namedtuple('CombinedUnit', 'decimal, binary')
 
@@ -107,11 +108,33 @@ def coerce_boolean(value):
             return False
         else:
             msg = "Failed to coerce string to boolean! (%r)"
-            raise ValueError(msg % value)
+            raise ValueError(format(msg, value))
     else:
         return bool(value)
 
 
+def coerce_pattern(value, flags=0):
+    """
+    Coerce strings to compiled regular expressions.
+
+    :param value: A string containing a regular expression pattern
+                  or a compiled regular expression.
+    :param flags: The flags used to compile the pattern (an integer).
+    :returns: A compiled regular expression.
+    :raises: :exc:`~exceptions.ValueError` when `value` isn't a string
+             and also isn't a compiled regular expression.
+    """
+    if is_string(value):
+        value = re.compile(value, flags)
+    else:
+        empty_pattern = re.compile('')
+        pattern_type = type(empty_pattern)
+        if not isinstance(value, pattern_type):
+            msg = "Failed to coerce value to compiled regular expression! (%r)"
+            raise ValueError(format(msg, value))
+    return value
+
+
 def format_size(num_bytes, keep_width=False, binary=False):
     """
     Format a byte count as a human readable file size.
@@ -208,7 +231,7 @@ def parse_size(size, binary=False):
                     return int(tokens[0] * (unit.binary.divider if binary else unit.decimal.divider))
     # We failed to parse the size specification.
     msg = "Failed to parse size! (input %r was tokenized as %r)"
-    raise InvalidSize(msg % (size, tokens))
+    raise InvalidSize(format(msg, size, tokens))
 
 
 def format_length(num_metres, keep_width=False):
@@ -277,7 +300,7 @@ def parse_length(length):
                     return tokens[0] * unit['divider']
     # We failed to parse the length specification.
     msg = "Failed to parse length! (input %r was tokenized as %r)"
-    raise InvalidLength(msg % (length, tokens))
+    raise InvalidLength(format(msg, length, tokens))
 
 
 def format_number(number, num_decimals=2):
@@ -464,7 +487,7 @@ def parse_timespan(timespan):
                     return float(tokens[0]) * unit['divider']
     # We failed to parse the timespan specification.
     msg = "Failed to parse timespan! (input %r was tokenized as %r)"
-    raise InvalidTimespan(msg % (timespan, tokens))
+    raise InvalidTimespan(format(msg, timespan, tokens))
 
 
 def parse_date(datestring):
@@ -535,7 +558,7 @@ def parse_date(datestring):
             return (year, month, day, 0, 0, 0)
     except Exception:
         msg = "Invalid date! (expected 'YYYY-MM-DD' or 'YYYY-MM-DD HH:MM:SS' but got: %r)"
-        raise InvalidDate(msg % datestring)
+        raise InvalidDate(format(msg, datestring))
 
 
 def format_path(pathname):
@@ -936,7 +959,7 @@ class InvalidDate(Exception):
     >>> parse_date('2013-06-XY')
     Traceback (most recent call last):
       File "humanfriendly.py", line 206, in parse_date
-        raise InvalidDate, msg % datestring
+        raise InvalidDate(format(msg, datestring))
     humanfriendly.InvalidDate: Invalid date! (expected 'YYYY-MM-DD' or 'YYYY-MM-DD HH:MM:SS' but got: '2013-06-XY')
     """
 
@@ -952,7 +975,7 @@ class InvalidSize(Exception):
     >>> parse_size('5 Z')
     Traceback (most recent call last):
       File "humanfriendly/__init__.py", line 267, in parse_size
-        raise InvalidSize(msg % (size, tokens))
+        raise InvalidSize(format(msg, size, tokens))
     humanfriendly.InvalidSize: Failed to parse size! (input '5 Z' was tokenized as [5, 'Z'])
     """
 
@@ -968,7 +991,7 @@ class InvalidLength(Exception):
     >>> parse_length('5 Z')
     Traceback (most recent call last):
       File "humanfriendly/__init__.py", line 267, in parse_length
-        raise InvalidLength(msg % (length, tokens))
+        raise InvalidLength(format(msg, length, tokens))
     humanfriendly.InvalidLength: Failed to parse length! (input '5 Z' was tokenized as [5, 'Z'])
     """
 
@@ -984,6 +1007,6 @@ class InvalidTimespan(Exception):
     >>> parse_timespan('1 age')
     Traceback (most recent call last):
       File "humanfriendly/__init__.py", line 419, in parse_timespan
-        raise InvalidTimespan(msg % (timespan, tokens))
+        raise InvalidTimespan(format(msg, timespan, tokens))
     humanfriendly.InvalidTimespan: Failed to parse timespan! (input '1 age' was tokenized as [1, 'age'])
     """
