@@ -1,7 +1,7 @@
 # Human friendly input/output in Python.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: May 18, 2017
+# Last Change: January 14, 2018
 # URL: https://humanfriendly.readthedocs.io
 
 """
@@ -46,20 +46,30 @@ Supported options:
     Convert a byte count (given as the integer BYTES) into a human readable
     string and print that string to standard output.
 
+  -b, --binary
+
+    Change the output of -s, --format-size to use binary multiples of bytes
+    (base-2) instead of the default decimal multiples of bytes (base-10).
+
   -t, --format-timespan=SECONDS
 
     Convert a number of seconds (given as the floating point number SECONDS)
     into a human readable timespan and print that string to standard output.
 
+  --parse-length=VALUE
+
+    Parse a human readable length (given as the string VALUE) and print the
+    number of metres to standard output.
+
   --parse-size=VALUE
 
     Parse a human readable data size (given as the string VALUE) and print the
     number of bytes to standard output.
 
-  --parse-length=VALUE
+  --demo
 
-    Parse a human readable length (given as the string VALUE) and print the
-    number of metres to standard output.
+    Demonstrate changing the style and color of the terminal font using ANSI
+    escape sequences.
 
   -h, --help
 
@@ -75,6 +85,8 @@ import sys
 
 # Modules included in our package.
 from humanfriendly import (
+    Spinner,
+    Timer,
     format_length,
     format_number,
     format_size,
@@ -82,19 +94,28 @@ from humanfriendly import (
     format_timespan,
     parse_length,
     parse_size,
-    Spinner,
-    Timer,
 )
-from humanfriendly.terminal import output, usage, warning
+from humanfriendly.tables import format_smart_table
+from humanfriendly.terminal import (
+    ANSI_COLOR_CODES,
+    ANSI_TEXT_STYLES,
+    HIGHLIGHT_COLOR,
+    ansi_strip,
+    ansi_wrap,
+    find_terminal_size,
+    output,
+    usage,
+    warning,
+)
 
 
 def main():
     """Command line interface for the ``humanfriendly`` program."""
     try:
-        options, arguments = getopt.getopt(sys.argv[1:], 'cd:hn:s:t:', [
-            'delimiter=', 'format-length=', 'format-number=', 'format-size=',
-            'format-table', 'format-timespan=', 'parse-length=',
-            'parse-size=', 'run-command', 'help',
+        options, arguments = getopt.getopt(sys.argv[1:], 'cd:l:n:s:bt:h', [
+            'run-command', 'format-table', 'delimiter=', 'format-length=',
+            'format-number=', 'format-size=', 'binary', 'format-timespan=',
+            'parse-length=', 'parse-size=', 'demo', 'help',
         ])
     except Exception as e:
         warning("Error: %s", e)
@@ -102,6 +123,7 @@ def main():
     actions = []
     delimiter = None
     should_format_table = False
+    binary = any(o in ('-b', '--binary') for o, v in options)
     for option, value in options:
         if option in ('-d', '--delimiter'):
             delimiter = value
@@ -116,11 +138,13 @@ def main():
         elif option in ('-n', '--format-number'):
             actions.append(functools.partial(print_formatted_number, value))
         elif option in ('-s', '--format-size'):
-            actions.append(functools.partial(print_formatted_size, value))
+            actions.append(functools.partial(print_formatted_size, value, binary))
         elif option == '--format-table':
             should_format_table = True
         elif option in ('-t', '--format-timespan'):
             actions.append(functools.partial(print_formatted_timespan, value))
+        elif option == '--demo':
+            actions.append(demonstrate_ansi_formatting)
         elif option in ('-h', '--help'):
             usage(__doc__)
             return
@@ -160,9 +184,9 @@ def print_formatted_number(value):
     output(format_number(float(value)))
 
 
-def print_formatted_size(value):
+def print_formatted_size(value, binary):
     """Print a human readable size."""
-    output(format_size(int(value)))
+    output(format_size(int(value), binary=binary))
 
 
 def print_formatted_table(delimiter):
@@ -187,3 +211,65 @@ def print_parsed_length(value):
 def print_parsed_size(value):
     """Parse a human readable data size and print the number of bytes."""
     output(parse_size(value))
+
+
+def demonstrate_ansi_formatting():
+    """Demonstrate the use of ANSI escape sequences."""
+    # First we demonstrate the supported text styles.
+    output('%s', ansi_wrap('Text styles:', bold=True))
+    styles = ['normal', 'bright']
+    styles.extend(ANSI_TEXT_STYLES.keys())
+    for style_name in sorted(styles):
+        options = dict(color=HIGHLIGHT_COLOR)
+        if style_name != 'normal':
+            options[style_name] = True
+        style_label = style_name.replace('_', ' ').capitalize()
+        output(' - %s', ansi_wrap(style_label, **options))
+    # Now we demonstrate named foreground and background colors.
+    for color_type, color_label in (('color', 'Foreground colors'),
+                                    ('background', 'Background colors')):
+        intensities = [
+            ('normal', dict()),
+            ('bright', dict(bright=True)),
+        ]
+        if color_type != 'background':
+            intensities.insert(0, ('faint', dict(faint=True)))
+        output('\n%s' % ansi_wrap('%s:' % color_label, bold=True))
+        output(format_smart_table([
+            [color_name] + [
+                ansi_wrap(
+                    'XXXXXX' if color_type != 'background' else (' ' * 6),
+                    **dict(kw.items() + [(color_type, color_name)])
+                ) for label, kw in intensities
+            ] for color_name in sorted(ANSI_COLOR_CODES.keys())
+        ], column_names=['Color'] + [
+            label.capitalize() for label, kw in intensities
+        ]))
+    # Demonstrate support for 256 colors as well.
+    demonstrate_256_colors(0, 7, 'standard colors')
+    demonstrate_256_colors(8, 15, 'high-intensity colors')
+    demonstrate_256_colors(16, 231, '216 colors')
+    demonstrate_256_colors(232, 255, 'gray scale colors')
+
+
+def demonstrate_256_colors(i, j, group=None):
+    """Demonstrate 256 color mode support."""
+    # Generate the label.
+    label = '256 color mode'
+    if group:
+        label += ' (%s)' % group
+    output('\n' + ansi_wrap('%s:' % label, bold=True))
+    # Generate a simple rendering of the colors in the requested range and
+    # check if it will fit on a single line (given the terminal's width).
+    single_line = ''.join(' ' + ansi_wrap(str(n), color=n) for n in range(i, j + 1))
+    lines, columns = find_terminal_size()
+    if columns >= len(ansi_strip(single_line)):
+        output(single_line)
+    else:
+        # Generate a more complex rendering of the colors that will nicely wrap
+        # over multiple lines without using too many lines.
+        width = len(str(j)) + 1
+        colors_per_line = columns / width
+        colors = [ansi_wrap(str(n).rjust(width), color=n) for n in range(i, j + 1)]
+        blocks = [colors[n:n + colors_per_line] for n in range(0, len(colors), colors_per_line)]
+        output('\n'.join(''.join(b) for b in blocks))
